@@ -25,18 +25,13 @@
 #include "oauth2plugin.h"
 #include "oauth2tokendata.h"
 
+#include <QJsonDocument>
 #include <QUrl>
+#include <QUrlQuery>
 #include <QNetworkRequest>
 #include <QNetworkReply>
 #include <QDateTime>
 
-#define USE_LIBQJSON (QT_VERSION < QT_VERSION_CHECK(5, 0, 0))
-
-#if USE_LIBQJSON
-#include <qjson/parser.h>
-#else
-#include <QJsonDocument>
-#endif
 
 using namespace SignOn;
 using namespace OAuth2PluginNS;
@@ -54,6 +49,7 @@ const int HTTP_STATUS_OK = 200;
 const QString AUTH_CODE = QString("code");
 const QString REDIRECT_URI = QString("redirect_uri");
 const QString RESPONSE_TYPE = QString("response_type");
+const QString STATE = QString("state");
 const QString USERNAME = QString("username");
 const QString PASSWORD = QString("password");
 const QString ASSERTION_TYPE = QString("assertion_type");
@@ -61,6 +57,7 @@ const QString ASSERTION = QString("assertion");
 const QString ACCESS_TOKEN = QString("access_token");
 const QString DISPLAY = QString("display");
 const QString EXPIRES_IN = QString("expires_in");
+const QString SCOPE = QString("scope");
 const QString TIMESTAMP = QString("timestamp");
 const QString GRANT_TYPE = QString("grant_type");
 const QString AUTHORIZATION_CODE = QString("authorization_code");
@@ -96,6 +93,7 @@ public:
     QString m_mechanism;
     OAuth2PluginData m_oauth2Data;
     QVariantMap m_tokens;
+    QString m_state;
     QString m_key;
     QString m_username;
     QString m_password;
@@ -133,6 +131,10 @@ void OAuth2Plugin::sendOAuth2AuthRequest()
     QUrl url(QString("https://%1/%2").arg(d->m_oauth2Data.Host()).arg(d->m_oauth2Data.AuthPath()));
     url.addQueryItem(CLIENT_ID, d->m_oauth2Data.ClientId());
     url.addQueryItem(REDIRECT_URI, d->m_oauth2Data.RedirectUri());
+    if (!d->m_oauth2Data.DisableStateParameter()) {
+        d->m_state = QString::number(qrand());
+        url.addQueryItem(STATE, d->m_state);
+    }
     if (!d->m_oauth2Data.ResponseType().isEmpty()) {
         url.addQueryItem(RESPONSE_TYPE,
                          d->m_oauth2Data.ResponseType().join(" "));
@@ -140,7 +142,6 @@ void OAuth2Plugin::sendOAuth2AuthRequest()
     if (!d->m_oauth2Data.Display().isEmpty()) {
         url.addQueryItem(DISPLAY, d->m_oauth2Data.Display());
     }
-    url.addQueryItem(QString("type"), d->m_mechanism);
     if (!d->m_oauth2Data.Scope().empty()) {
         QString separator = QLatin1String(" ");
 
@@ -154,7 +155,7 @@ void OAuth2Plugin::sendOAuth2AuthRequest()
         }
 
         // Passing list of scopes
-        url.addQueryItem(QString("scope"), d->m_oauth2Data.Scope().join(separator));
+        url.addQueryItem(SCOPE, d->m_oauth2Data.Scope().join(separator));
     }
     TRACE() << "Url = " << url.toString();
     SignOn::UiSessionData uiSession;
@@ -265,6 +266,15 @@ void OAuth2Plugin::process(const SignOn::SessionData &inData,
         tokens.setTokens(d->m_tokens);
         emit store(tokens);
         TRACE() << d->m_tokens;
+    } else if (d->m_oauth2Data.ForceTokenRefresh()) {
+        // remove only the access token, not the refresh token
+        QVariantMap storedData = d->m_tokens.value(d->m_key).toMap();
+        storedData.remove(TOKEN);
+        d->m_tokens.insert(d->m_key, storedData);
+        OAuth2TokenData tokens;
+        tokens.setTokens(d->m_tokens);
+        Q_EMIT store(tokens);
+        TRACE() << "Clearing access token" << d->m_tokens;
     }
 
     //get provided token data if specified
@@ -326,18 +336,7 @@ void OAuth2Plugin::userActionFinished(const SignOn::UiSessionData &data)
 {
     Q_D(OAuth2Plugin);
 
-    TRACE();
-
-    if (data.QueryErrorCode() != QUERY_ERROR_NONE) {
-        TRACE() << "userActionFinished with error: " << data.QueryErrorCode();
-        if (data.QueryErrorCode() == QUERY_ERROR_CANCELED)
-            emit error(Error(Error::SessionCanceled, QLatin1String("Cancelled by user")));
-        else
-            emit error(Error(Error::UserInteraction,
-                             QString("userActionFinished error: ")
-                             + QString::number(data.QueryErrorCode())));
-        return;
-    }
+    if (handleUiErrors(data)) return;
 
     TRACE() << data.UrlResponse();
 
@@ -352,21 +351,31 @@ void OAuth2Plugin::userActionFinished(const SignOn::UiSessionData &data)
     if (d->m_mechanism == USER_AGENT) {
         // Response should contain the access token
         OAuth2PluginTokenData respData;
-        QString fragment;
         if (url.hasFragment()) {
-            fragment = url.fragment();
-            QStringList list = fragment.split(QRegExp("&|="), QString::SkipEmptyParts);
-            for (int i = 1; i < list.count(); i += 2) {
-                if (list.at(i - 1) == ACCESS_TOKEN) {
-                    respData.setAccessToken(list.at(i));
-                }
-                else if (list.at(i - 1) == EXPIRES_IN) {
-                    respData.setExpiresIn(QString(list.at(i)).toInt());
-                }
-                else if (list.at(i - 1) == REFRESH_TOKEN) {
-                    respData.setRefreshToken(list.at(i));
+            QString state;
+            respData.setScope(d->m_oauth2Data.Scope());
+            QUrlQuery fragment(url.fragment());
+            typedef QPair<QString, QString> StringPair;
+            Q_FOREACH(const StringPair &pair, fragment.queryItems()) {
+                if (pair.first == ACCESS_TOKEN) {
+                    respData.setAccessToken(pair.second);
+                } else if (pair.first == EXPIRES_IN) {
+                    respData.setExpiresIn(pair.second.toInt());
+                } else if (pair.first == REFRESH_TOKEN) {
+                    respData.setRefreshToken(pair.second);
+                } else if (pair.first == STATE) {
+                    state = pair.second;
+                } else if (pair.first == SCOPE) {
+                    respData.setScope(pair.second.split(' ', QString::SkipEmptyParts));
                 }
             }
+            if (!d->m_oauth2Data.DisableStateParameter() &&
+                state != d->m_state) {
+                Q_EMIT error(Error(Error::NotAuthorized,
+                                   QString("'state' parameter mismatch")));
+                return;
+            }
+
             if (respData.AccessToken().isEmpty()) {
                 emit error(Error(Error::NotAuthorized, QString("Access token not present")));
             } else {
@@ -386,6 +395,12 @@ void OAuth2Plugin::userActionFinished(const SignOn::UiSessionData &data)
         // 4. Refresh Token (refresh_token)
         QUrl newUrl;
         if (url.hasQueryItem(AUTH_CODE)) {
+            if (!d->m_oauth2Data.DisableStateParameter() &&
+                d->m_state != url.queryItemValue(STATE)) {
+                Q_EMIT error(Error(Error::NotAuthorized,
+                                   QString("'state' parameter mismatch")));
+                return;
+            }
             QString code = url.queryItemValue(AUTH_CODE);
             newUrl.addQueryItem(GRANT_TYPE, AUTHORIZATION_CODE);
             newUrl.addQueryItem(AUTH_CODE, code);
@@ -421,9 +436,52 @@ void OAuth2Plugin::userActionFinished(const SignOn::UiSessionData &data)
     }
 }
 
+QVariantMap OAuth2Plugin::parseReply(const QByteArray &contentType,
+                                     const QByteArray &replyContent)
+{
+    typedef QVariantMap (OAuth2Plugin::*Parser)(const QByteArray &replyContent);
+    Parser preferredParser;
+    Parser fallbackParser;
+
+    QVariantMap map;
+
+    // Handling application/json content type
+    if (contentType.startsWith(CONTENT_APP_JSON)) {
+        TRACE() << "application/json content received";
+        preferredParser = &OAuth2Plugin::parseJSONReply;
+        fallbackParser = &OAuth2Plugin::parseTextReply;
+    }
+    // Added for facebook Graph API's (handling text/plain content type)
+    else if (contentType.startsWith(CONTENT_TEXT_PLAIN) ||
+               contentType.startsWith(CONTENT_APP_URLENCODED)) {
+        TRACE() << contentType << "content received";
+        preferredParser = &OAuth2Plugin::parseTextReply;
+        fallbackParser = &OAuth2Plugin::parseJSONReply;
+    } else {
+        TRACE() << "Unsupported content type received: " << contentType;
+        Q_EMIT error(Error(Error::OperationFailed,
+                           QString("Unsupported content type received")));
+        return map;
+    }
+
+    map = (this->*preferredParser)(replyContent);
+    if (Q_UNLIKELY(map.isEmpty())) {
+        TRACE() << "Parse failed, trying fallback parser";
+        map = (this->*fallbackParser)(replyContent);
+        if (Q_UNLIKELY(map.isEmpty())) {
+            TRACE() << "Parse failed";
+            Q_EMIT error(Error(Error::NotAuthorized,
+                               QString("No access token found")));
+        }
+    }
+    return map;
+}
+
 // Method to handle responses for OAuth 2.0 requests
 void OAuth2Plugin::serverReply(QNetworkReply *reply)
 {
+    Q_D(OAuth2Plugin);
+
     QByteArray replyContent = reply->readAll();
     TRACE() << replyContent;
 
@@ -437,55 +495,39 @@ void OAuth2Plugin::serverReply(QNetworkReply *reply)
 
     // Handling 200 OK response (HTTP_STATUS_OK) WITH content
     if (reply->hasRawHeader(CONTENT_TYPE)) {
-
-        // Handling application/json content type
-        if (reply->rawHeader(CONTENT_TYPE).startsWith(CONTENT_APP_JSON)) {
-            TRACE()<< "application/json content received";
-            QVariantMap map = parseJSONReply(replyContent);
-            QByteArray accessToken = map["access_token"].toByteArray();
-            QVariant expiresIn = map["expires_in"];
-            QByteArray refreshToken = map["refresh_token"].toByteArray();
-
-            if (accessToken.isEmpty()) {
-                TRACE()<< "Access token is empty";
-                emit error(Error(Error::NotAuthorized,
-                                 QString("Access token is empty")));
-            }
-            else {
-                OAuth2PluginTokenData response;
-                response.setAccessToken(accessToken);
-                response.setRefreshToken(refreshToken);
-                response.setExpiresIn(expiresIn.toInt());
-                storeResponse(response);
-                emit result(response);
-            }
+        QVariantMap map = parseReply(reply->rawHeader(CONTENT_TYPE), replyContent);
+        if (Q_UNLIKELY(map.isEmpty())) {
+            // The error has already been delivered
+            return;
         }
-        // Added to test with facebook Graph API's (handling text/plain content type)
-        else if (reply->rawHeader(CONTENT_TYPE).startsWith(CONTENT_TEXT_PLAIN) ||
-                 reply->rawHeader(CONTENT_TYPE).startsWith(CONTENT_APP_URLENCODED)) {
-            TRACE()<< reply->rawHeader(CONTENT_TYPE) << "content received";
-            QMap<QString,QString> map = parseTextReply(replyContent);
-            QByteArray accessToken = map["access_token"].toAscii();
-            QByteArray expiresIn = map["expires"].toAscii();
-            QByteArray refreshToken = map["refresh_token"].toAscii();
-
-            if (accessToken.isEmpty()) {
-                TRACE()<< "Access token is empty";
-                emit error(Error(Error::NotAuthorized,
-                                 QString("Access token is empty")));
-            }
-            else {
-                OAuth2PluginTokenData response;
-                response.setAccessToken(accessToken);
-                response.setRefreshToken(refreshToken);
-                response.setExpiresIn(expiresIn.toInt());
-                storeResponse(response);
-                emit result(response);
-            }
+        QByteArray accessToken = map["access_token"].toByteArray();
+        int expiresIn = map["expires_in"].toInt();
+        if (expiresIn == 0) {
+            // Facebook uses just "expires" as key
+            expiresIn = map["expires"].toInt();
         }
-        else {
-            TRACE()<< "Unsupported content type received: " << reply->rawHeader(CONTENT_TYPE);
-            emit error(Error(Error::OperationFailed, QString("Unsupported content type received")));
+        QByteArray refreshToken = map["refresh_token"].toByteArray();
+
+        QStringList scope;
+        if (map.contains(SCOPE)) {
+            QString rawScope = QString::fromUtf8(map[SCOPE].toByteArray());
+            scope = rawScope.split(' ', QString::SkipEmptyParts);
+        } else {
+            scope = d->m_oauth2Data.Scope();
+        }
+
+        if (accessToken.isEmpty()) {
+            TRACE()<< "Access token is empty";
+            Q_EMIT error(Error(Error::NotAuthorized,
+                               QString("Access token is empty")));
+        } else {
+            OAuth2PluginTokenData response;
+            response.setAccessToken(accessToken);
+            response.setRefreshToken(refreshToken);
+            response.setExpiresIn(expiresIn);
+            response.setScope(scope);
+            storeResponse(response);
+            emit result(response);
         }
     }
     // Handling 200 OK response (HTTP_STATUS_OK) WITHOUT content
@@ -495,6 +537,18 @@ void OAuth2Plugin::serverReply(QNetworkReply *reply)
     }
 }
 
+bool OAuth2Plugin::handleNetworkError(QNetworkReply *reply,
+                                      QNetworkReply::NetworkError err)
+{
+    if (err >= QNetworkReply::ContentAccessDenied) {
+        QByteArray replyContent = reply->readAll();
+        TRACE() << replyContent;
+        handleOAuth2Error(replyContent);
+        return true;
+    }
+    return BasePlugin::handleNetworkError(reply, err);
+}
+
 void OAuth2Plugin::handleOAuth2Error(const QByteArray &reply)
 {
     Q_D(OAuth2Plugin);
@@ -558,8 +612,6 @@ void OAuth2Plugin::handleOAuth2Error(const QByteArray &reply)
 
 void OAuth2Plugin::refreshOAuth2Token(const QString &refreshToken)
 {
-    Q_D(OAuth2Plugin);
-
     TRACE() << refreshToken;
     QUrl url;
     url.addQueryItem(GRANT_TYPE, REFRESH_TOKEN);
@@ -639,28 +691,22 @@ void OAuth2Plugin::storeResponse(const OAuth2PluginTokenData &response)
     TRACE() << d->m_tokens;
 }
 
-const QVariantMap OAuth2Plugin::parseJSONReply(const QByteArray &reply)
+QVariantMap OAuth2Plugin::parseJSONReply(const QByteArray &reply)
 {
     TRACE();
-    bool ok = false;
-#if USE_LIBQJSON
-    QJson::Parser parser;
-    QVariant tree = parser.parse(reply, &ok);
-#else
     QJsonDocument doc = QJsonDocument::fromJson(reply);
-    ok = !doc.isEmpty();
+    bool ok = !doc.isEmpty();
     QVariant tree = doc.toVariant();
-#endif
     if (ok) {
         return tree.toMap();
     }
     return QVariantMap();
 }
 
-const QMap<QString, QString> OAuth2Plugin::parseTextReply(const QByteArray &reply)
+QVariantMap OAuth2Plugin::parseTextReply(const QByteArray &reply)
 {
     TRACE();
-    QMap<QString, QString> map;
+    QVariantMap map;
     QList<QByteArray> items = reply.split('&');
     foreach (QByteArray item, items) {
         int idx = item.indexOf("=");
